function ARMA_Asy(pi_true, m, pi_H)
%This function simulates the asymptotic distributions of the estimators and the t and QLR statistics, calculates the asymptotic coverage probabilities of the standard, least favorable, and type 2 robust CIs with different kappa values. 
%by Donald Andrews and Xu Cheng
%Inputs:
%pi_true: true value of pi
%m: number of simulation repetitions
%pi_H: upper bound of the parameter space for pi

pi_L = -pi_H;
TN1 = round(pi_true*100);
%% ARMA model Y_t = (pi_0+beta_0)*Y_t-1+e_t-pi_0*e_t-1
zeta_true= 1; % this is the variance of error: e
alpha = 0.05; %nominal size of the test

%grids for b values
upp1 = 5;
upp2 = 10;
upp3 = 20;
stepsi1 = 0.1;
stepsi2 = 0.2;
stepsi3 = 1;
b1 = 0:stepsi1:upp1;
b2 = upp1+stepsi2:stepsi2:upp2;
b3 = upp2+stepsi3:stepsi3:upp3;
b_V = [b1,b2,b3]';
rb = rows(b_V);

bmax_abs = 17; %This is based on some preliminary simulation. bmax is the index of b over which the LF crit value is achieved (over all pi_true)
bmax_qlr = 30;
%They could be replaced by bmax0_abs and bmax0_qlr generated below,
%respectively. These are null-imposed versions and do not require
%preliminary simulation.

TbM= zeros(rb,3,m); TbM_qlr = zeros(rb,1,m); TbM_qlr_pi = zeros(rb,1,m);
TbM_piest= zeros(rb,1,m);  TbM_se = zeros(rb,3,m);

%quantities used in the calculation of unrestricted and restricted
%criterion functions below
n=2000; %sample size to approx population moments
lz = 100; %lags truncated
col_Ys = n-lz-1;
Y_s1 = zeros(lz, col_Ys);
power = 1:lz;
expt1 = sqrt(2)*zeta_true;
expt2 = 1/2*zeta_true^(-2)*expt1;

opt = optimset('LargeScale','off', ...
               'HessUpdate','bfgs', ...
               'Algorithm', 'active-set',...
               'Hessian','off', ...
               'GradObj','off', ...
               'DerivativeCheck','off',...
               'Display', 'off');

%% Simulation Repetition Starts
z=1;
while z<=m;           
e = sqrt(zeta_true)*randn(n+1,1);
for i = 1:col_Ys
    Y_s1(:,i) = e(n-(i-1):-1:(n-(i-1)-lz+1),:);
end
Z1 = randn(1,1);
Z2 = randn(lz,1);

%% 
parfor k = 1:rb
    
b = b_V(k,1);
r_psi = [1,0]; % slector matrix that select beta out of psi = [beta, zeta].
Q = cf_asy_unres(Z1, Z2, power, expt2, zeta_true, pi_true, b, pi_true);
Q_qlr = cf_asy_qlr(Z1, Z2, power, expt2, zeta_true, pi_true, b, pi_true, r_psi);

%grid search to find starting values for the optimization of Q and Q_qlr
pi_star = pi_true;
pi_qlr_star = pi_true;
for i=0:0.02:1
    pi =pi_L+i*(pi_H-pi_L);
    Q1 = cf_asy_unres(Z1, Z2, power, expt2, zeta_true, pi_true, b, pi);
    Q_qlr1 = cf_asy_qlr(Z1, Z2, power, expt2, zeta_true, pi_true, b, pi, r_psi);
    if Q1 <Q
       pi_star = pi;
       Q= Q1;
    end
    if Q_qlr1 < Q_qlr
        pi_qlr_star = pi;
        Q_qlr = Q_qlr1;
    end
end

%unrestricted estimator of pi
[pi_star Q_unres] = fmincon(@(pi)cf_asy_unres (Z1, Z2, power, expt2, zeta_true, pi_true, b, pi),pi_star, [], [], [], [], pi_L, pi_H, [], opt);
%restricted estimator of pi wben tbe restriction if beta = v
[pi_qlr_star Q_res_beta]= fmincon(@(pi)cf_asy_qlr(Z1, Z2, power, expt2, zeta_true, pi_true, b, pi, r_psi), pi_qlr_star, [], [], [], [], pi_L, pi_H, [], opt);

%% QLR statistic for beta
qlr = 2*(Q_res_beta - Q_unres);

%% QLR statistics for pi
Q_qlr_0 = cf_asy_unres (Z1, Z2, power, expt2, zeta_true, pi_true, b, pi_true); % restricted criterion function at tbe true value of pi
qlr_pi = 2*(Q_qlr_0 - Q_unres); 

%% Calucate J matrix, standard error, and t statistic

Y_s2 = pi_star.^(power-1)*Y_s1;
Y_s3 = (power.*pi_star.^(power-1))*Y_s1; 
Y_s2n = Y_s2(:,1:end-1);
Y_s3n = Y_s3(:,2:end);
Y_e1 = mean(Y_s2.^2,2);
Y_e2 = mean(Y_s3.^2,2);
Y_e3 = mean(Y_s2n.*Y_s3n,2);

rot = [0,0,1;
       0,0,0;
       1,0,0];

J1 = 1/zeta_true*Y_e1;
J2 = 1/(2*zeta_true^2);
J3 = 1/zeta_true*Y_e2;
J = diag([J1,J2,J3])+1/zeta_true*Y_e3*rot;

Sigma = inv(J);
se = sqrt(diag(Sigma));

% G
G1 = pi_star.^(power-1)*Z2;
G2 = expt2*Z1;
G = [G1;G2];
% H 
H1 = 1-pi_star^2;
H2 = 2*zeta_true^2;
H_inv = diag([H1,H2]);
% K
K1 = 1/(1-pi_true*pi_star)*b;
K = -[K1;0];
% t statistic
tao = -H_inv*(G+K) - [b;0];
taopi = abs((tao(1,:)+b))*(pi_star-pi_true);
process = [tao; taopi];
t = process./se;

%%%%% end of t statistics %%%%%%%%%

%% stack results together for simulated quantiles
TbM_qlr(k,:,z) = qlr;
TbM_qlr_pi(k,:,z) = qlr_pi;
TbM_piest(k,:,z) = pi_star;
TbM(k,:,z) = t;
TbM_se(k,:,z) = se;

end  % finish tbe loop over b

z = z+1;
end  % finish the loop lover z

%% t Quantile
TbM_abs = abs(TbM);
Tb_sort_abs = sort(TbM_abs, 3);
qb_abs = Tb_sort_abs(:,:,ceil(m*(1-alpha)));

%% QLR Quantile for beta and pi
Tb_sort_qlr = sort(TbM_qlr, 3);
qb_qlr = Tb_sort_qlr(:,:,ceil(m*(1-alpha))); 
Tb_sort_qlr_pi = sort(TbM_qlr_pi, 3);
qb_qlr_pi = Tb_sort_qlr_pi(:,:,ceil(m*(1-alpha))); 

%% asymptotic coverage probabilities for standard, least favorable, and type-II robust CIs for pi and rho
% asymptotic process of the ICS statistic
B = zeros(size(TbM));
for i = 1:rb
    B(i,:,:) = b_V(i);
end
A = TbM.*TbM_se;
A = A + B; % centered at 0
A = A./TbM_se; % normalize
A = A(:,1,:); % select the studentized statistic for beta for identification-category-selection
A = abs(A); % abs value

%parameters in robust CIs
K_V = [0,0.5,1,1.5,1.75,2,2.25,2.5,2.75,3,4,5,6,8,10];
c = 0.5;
ind =1;

% symmetric two-sided t CIs
W = abs(TbM(:,3,:));
std_cv = icdf('normal', 1-alpha/2, 0,1);
[LF_cv_abs,bmax0_abs] = max(qb_abs(:,3)); %Alternatively, we could use bmax0 in place bmax below.
adj = 10; %10 grids correspond to b=1 in the region where bmax is achieved in cases where adjustment is needed and makes a difference

tao_V_abs =[];
for j = 1:cols(K_V);
    K = K_V(j);
    [tao, tao2_V]= sizecorr(W, A, LF_cv_abs, std_cv, K,c,ind,bmax_abs,adj,alpha); %tao2_V is not used in calculation, but the values can be used to check for what b values size corrections with tao2 is needed.
    tao_V_abs = [tao_V_abs; tao];
end
covprob_asy_abs = covprob(W, A, std_cv, LF_cv_abs, K_V, tao_V_abs, c,ind);

% QLR CIs
W = TbM_qlr_pi;
std_cv =  icdf('chi2', 1-alpha, 1);
[LF_cv_qlr,bmax0_qlr] = max(qb_qlr_pi);

tao_V_qlr =[];
for j = 1:cols(K_V);
    K = K_V(j);
    [tao, tao2_V]= sizecorr(W, A, LF_cv_qlr, std_cv, K,c,ind,bmax_qlr,adj,alpha); %tao2_V is not used in calculation, but the values can be used to check for what b values size corrections with tao2 is needed.
    tao_V_qlr = [tao_V_qlr; tao];
end
covprob_asy_qlr =  covprob(W, A, std_cv, LF_cv_qlr, K_V, tao_V_qlr, c,ind);


save(sprintf('ARMA_Asy_%d.mat',TN1),'TbM','TbM_piest',...
     'TbM_qlr','TbM_qlr_pi', 'TbM_se',...
     'qb_abs', 'qb_qlr','qb_qlr_pi','b_V',...
     'covprob_asy_abs','covprob_asy_qlr','tao_V_abs','tao_V_qlr',...
     'pi_true','LF_cv_abs','LF_cv_qlr','K_V');




















