function  [pp, power] = LIML_power(k, rho, m)
% This programs compute the power functions of the CLR CI, Type 2 NI Robust
% CI, and those based on the LM and AR statistics.
% INPUTS:
% k: number of IV
% rho: corr in error cov matrix
% lambda_true: concentration parameter
% m: number of mulation repetitions
% OUTPUTS:
% pp: grids of pi*sqrt(lambda) over which power is computed
% power: its five columns are powers of tests based on the (1) CLR statistic, (2) type 2 robust
% test with null-imposed ICS, (3) type 2 robust test with unrestricted ICS, (4) LM
% statistic, and (5) AR statistic.

%% CLR Critical Values from Andrews, Moreira, and Stock (2006)
load ('CLR_CV.mat'); % CLR critical values from 

%% parameter set-up
K_V =[0, 0.5, 1, 1.5, 2, 2.5, 3]; %Kappa values considered for the type 2 robust CI
alpha = 0.05; % nominal size
pi_null = 0; % null value of pi
omg11 = 1;
omg12 = rho;
omg22 =1;
Omega = [omg11, omg12; omg12, omg22]; % covariance matrix of reduced-from errors


% Robust CI parameters
std_cv = 3.84;
c = 2; % the constant in the transition function for robust CI
ind = 1; % the parametric function for the transition function, 1 is logistic
UMatrix = randn(k,2*m);

%% simulate LF critical value and size correction factors
[LF_cv, tao_null_V, tao_unres_V,covprob_null, covprob_unres, LR_sort, cv_lam, rl,lambdaV] = LIML_LF(k, rho, K_V, m,UMatrix);
nk = length(K_V);
tao_null_V = reshape(tao_null_V,nk,2);
tao_unres_V = reshape(tao_unres_V,nk,2);
covprob_null = reshape(covprob_null,rl,nk+2);
covprob_unres = reshape(covprob_unres,rl,nk+2);
LR_sort = reshape(LR_sort,rl,1,m);
cv_lam = cv_lam';

LV = [2,5,10,20]; 
nlv = length(LV);
pp = -6:0.2:6; % bounds -6 and 6 are from AMS, pp is the X-axis of power plot
z = length(pp); % number of the true values of pi considered

LR_M = zeros(z, nlv, m);
ICS_null_M = zeros(z, nlv ,m);
ICS_unres_M = zeros(z, nlv, m);
Qt_M = zeros(z, nlv, m);
Qs_M = zeros(z, nlv, m);
Qst_M = zeros(z, nlv, m);
%% Power Calculation for Type 2 Robust CI
%plot power as a function of pi*sqrt(lambda)

for t = 1: nlv; %loop over different true values of lambda
    lambda_true = LV(t);
pi_V = pp/sqrt(lambda_true); % null values of pi for power calculation, a vector
b_t = ones(k,1)*sqrt(lambda_true/k); % true value of b, this is strength of IV
    
for j = 1:m  % over different repetitions
    US = UMatrix(:,j); % in distn of S
    UT = UMatrix(:,j+m); % in distn of T  

  
parfor i = 1:z    
    pi_t = pi_V(i);
    [S, T] = LIML_ST (Omega, b_t, pi_t, pi_null, US, UT); 
    Qs = S'*S;
    Qt = T'*T;
    Qst = S'*T;
    LR = 1/2*(Qs - Qt + sqrt((Qs-Qt)^2+4*Qst^2));
    ICS_null = sqrt(T'*T/k);
    ICS_unres = LIML_ICS(S, T, k, pi_null, Omega);
    
    LR_M(i,t,j) = LR;
    ICS_null_M (i,t,j) = ICS_null;
    ICS_unres_M (i,t,j) = ICS_unres;
    Qt_M(i,t,j) = Qt;
    Qs_M (i,t,j) = Qs;
    Qst_M(i,t,j) = Qst;

end  %end of loop over true values of pi 

end  % end of different m repitations

end  % end the true value of lambda

nk = cols(K_V);
nrj_null = zeros(z,nlv,nk);
nrj_unres = zeros (z,nlv,nk);

for t = 1:nlv
parfor i = 1:z
    LR = LR_M(i,t,:); % for differnt true values of pi
    ICS_null = ICS_null_M(i,t,:);
    ICS_unres = ICS_unres_M(i,t,:);
    
    %% prob of rejecting the null
    nrj0 =  1-covprob(LR, ICS_null, std_cv, LF_cv, K_V, tao_null_V, c,ind);
    nrj_null(i,t,:) = nrj0(3:end);% first two are standard and LF 
    nrj0 = 1-covprob(LR, ICS_unres, std_cv, LF_cv, K_V, tao_unres_V, c,ind);
    nrj_unres(i,t,:) =  nrj0(3:end);
end
end

% average over two dimensions
nrj_null_avg = mean(nrj_null,1);
nrj_null_avg = mean(nrj_null_avg,2);
[maxsum_null, nrj_null_ind] = max(nrj_null_avg,[],3); % choose kappa with highest power
kappa_null = K_V(nrj_null_ind);
nrj_null_max = nrj_null(:,:,nrj_null_ind); % corresponding to opt kappa
covprob_null = covprob_null(:,2+nrj_null_ind); %size with the kappa selected

nrj_unres_avg = mean(nrj_unres,1);
nrj_unres_avg = mean(nrj_unres_avg,2);
[maxsum_unres, nrj_unres_ind] = max(nrj_unres_avg,[],3);
kappa_unres = K_V(nrj_unres_ind);
nrj_unres_max = nrj_unres(:,:,nrj_unres_ind);
covprob_unres = covprob_unres(:,2+nrj_unres_ind); %size with the kappa selected
%% Power Calculation of CLR
% CLR_CV is the matrix of CLR crit vals from AMS appendix
% the first row is the value of log(QT/k)
% the second to fifth columns are crit vals for k=2,5,10,20
%% specify the column of the crit val
% Note: the first column is ln(Qt/k)
if k==2
    cv_col =2;
elseif k==5
    cv_col =3;
elseif k==10
    cv_col=4;
else cv_col=5;
end

%% specify the row column of crit val
Adj_Qt_M = log(Qt_M/k) ; % this is the rows of CLR_CV in AMS
z_cv = rows(CLR_CV); % row numbers of crit val matrix
% the first row of CLR_CV is less than -5, second row is 5, ..., last row is larger
% than 6
cv_row = (Adj_Qt_M - (-5))/0.1+2; % plus 2 because -5 is in the second row
cv_row_top = (6 - (-5))/0.1+2; % largest one for interpolation
cv_row_L = floor(cv_row);
R1 = cv_row - cv_row_L; % a matrix of weight for interpolation

%% calculate the robust crit val
crit_CLR_M = zeros(z,nlv,m);
for t = 1:nlv
for i = 1:z
    for j = 1:m
        cv_L = cv_row_L(i,t,j); % the row number
        if cv_L < 2;
            crit_CLR_M(i,t,j)=CLR_CV(1,cv_col);
        elseif cv_L >= cv_row_top;
            crit_CLR_M(i,t,j)=CLR_CV(z_cv,cv_col);
        else
        cv_H = cv_L+1;        
        crit_CLR_M(i,t,j) = CLR_CV(cv_L,cv_col)+ R1(i,t,j)*(CLR_CV(cv_H,cv_col)-CLR_CV(cv_L,cv_col));
        end    
    end
end
end
%% CLR CI Power Function
nrj_OPT_M = mean(LR_M>crit_CLR_M,3);

%% AR Power Function
AR_M = Qs_M/k;
AR_cv  =  chi2inv(1-alpha,k)/k;
nrj_AR_M = mean(AR_M>AR_cv,3);

%% LM Power Function
LM_M = Qst_M.^2./Qt_M;
LM_cv = 3.84;
nrj_LM_M = mean(LM_M>LM_cv,3);
%% plot
power = [nrj_OPT_M, nrj_null_max, nrj_unres_max, nrj_LM_M, nrj_AR_M];

save(sprintf('LIML_%d_%d', k,rho*100),'pp','power','covprob_null',...
    'covprob_unres', 'LR_sort', 'cv_lam','kappa_unres','kappa_null',...
    'LF_cv', 'tao_null_V', 'tao_unres_V','nrj_null','nrj_unres');









    
    
    
    


    
    












