function [A_F, Q_FCP_M,t_FCP_M]= MA_FCP(pi_true, pi_null_V, b_VV, n, m, pi_H,rho_H)
% For a given true value of the MA parameter, this program simulates the
% finite-sample distribution of the t and QLR statistics under various
% null values of the AR parameter that are different from the true value.
% by Donald Andrews and Xu Cheng
% INPUTS: 
% pi_true is the true value of the MA coef
% pi_null_V: a vector of null values for pi
% b_VV: b values considered
% m is number of simulation repetitions
% n is the finite-sample size
% pi_H is the upper bound of the opt space for MA coef
% rho_H is the upper bound of the opt space for AR coef
% OUTPUTS:
% A_F: ICS statistic
% QLR_FCP_M: QLR statistic when the null is different from the true value
% t_FCP_M: t statistic when the null is different from the true value



pi_L= -pi_H;
rho_L = -rho_H;

%% parameter set-up
zeta_true= 1; % this is the variance of error: e
opt = optimset('LargeScale','off', ...
               'HessUpdate','bfgs', ...
               'Algorithm', 'active-set',...
               'Hessian','off', ...
               'GradObj','on', ...
               'DerivativeCheck','off',...
               'Display', 'off');
    
rb=length(b_VV);

%matrix used in the optimization below
A = [1,1;
    -1,-1;
    0,1;
    0,-1;];
bb = [rho_H; -rho_L; pi_H; -pi_L];

nd = 200; % get rid of initial effect

TbM_F = zeros(rb,3,m);  A_F = zeros(rb,1,m); TbM_qlr_F = zeros(rb,1,m); 
TbM_qlr_pi_F = zeros(rb,1,m); Theta_M=zeros(rb,3,m); 
Q_FCP_M = zeros(rb,length(pi_null_V),m);
t_FCP_M = zeros(rb,length(pi_null_V),m);


%% Simulation Repetition Starts
z=1;
while z<=m;
%% DGP %%%
e = sqrt(zeta_true)*randn(n+nd,1);
error = e(2:end)-pi_true*e(1:end-1);
error2 = [e(1);error];
rnd = rand(10,2); % random number for starting points in global minimization

for k = 1:rb
    b = b_VV(k,1);
    beta_true = -b/sqrt(n);
    rho_true = pi_true + beta_true;
    
    Y = zeros(n+nd,1);
    for t=1:n+nd
        power = t-1:-1:0;      
        rho_power = [rho_true.^power,zeros(1,n+nd-t)];
        Y(t)=rho_power*error2;
    end
        
lz=100; %lags beyond lz are truncated for computational ease
Y = Y(end:-1:end-n-lz,1);
Y_lag = zeros(lz,n);
for i =1:n
    Y_lag(:,i) = Y(i+1:i+lz); %lags for Y_i, a lz by 1 vector
end

Y_up = Y(1:n);
pow = 1:lz;
theta_true = [beta_true, zeta_true, pi_true];
 

%% resticted estimator pi = pi_true
b_L = rho_L - pi_true;
b_H = rho_H - pi_true;
[theta_qlr_pi, Q_pi] = fmincon(@(beta)cf_beta(Y_up, Y_lag, pow, beta, pi_true), beta_true, [], [], [], [], b_L, b_H, [], opt);
%starting value is beta_true because beta is strongly-identified and
%consistely estimatable



%% Unrestricted Estimation
theta_star_true = [beta_true, pi_true];
[theta_star, Q_unres] = fmincon(@(theta)cf_theta(Y_up, Y_lag, pow,theta),theta_star_true, A, bb, [], [], [], [], [], opt);
% optimization with random starting points
for j =1:10;
    pi0 = pi_L+rnd(j,1)*(pi_H-pi_L);
    beta0 = rho_L + rnd(j,2)*(rho_H-rho_L)-pi0;
    theta0=[beta0,pi0];
    [theta_star1 Q_unres1] = fmincon(@(theta)cf_theta(Y_up, Y_lag, pow,theta),theta0, A, bb, [], [], [], [], [], opt);
        if Q_unres1 < Q_unres
           theta_star = theta_star1;
           Q_unres = Q_unres1;
        end
end
beta_star = theta_star(1);
zeta_star = Q_unres/n; % Q = sum square errors
pi_star = theta_star(2);
theta_star2 = [beta_star, zeta_star, pi_star]; %the whole vector of estimated parameters


%% QLR statistic for pi
qlr_pi = n*(log(Q_pi) - log(Q_unres));
if qlr_pi<0 %This only happens rarely due to computational errors
    qlr_pi=0;
end

%% t statistic
pi_pow = pi_star.^(pow-1);
Y_lag_pi = (pi_pow*Y_lag)';
p2=pow(2:end);
pi_pow_j = [0,(p2-1).*pi_star.^(p2-2)];
Y_der_pi = (pi_pow_j*Y_lag)';

J1 = Y_lag_pi'*Y_lag_pi/n/zeta_star;
J2 = 1/2/zeta_star^2;
J3 = Y_der_pi'*Y_der_pi/n/zeta_star;
J4 = Y_lag_pi'*Y_der_pi/n/zeta_star;
rot = [0,0,1;
       0,0,0;
       1,0,0];
J = diag([J1,J2,J3])+J4*rot;

se = sqrt(diag(inv(J)))'/sqrt(n)*diag([ones(1,2),1/abs(beta_star)]);
t = (theta_star2-theta_true) ./se;
pre_t = abs(theta_star2./se); % centered at 0 rather than the true value. This is used for identification-categroy selection
se_pi = se(3);
%% FCP 
% resticted estimator pi = pi_null
b_L = rho_L - pi_true;
b_H = rho_H - pi_true;
Q_FCP_V = zeros(1,length(pi_null_V));
t_FCP_V = zeros(1,length(pi_null_V));
parfor q = 1:length(pi_null_V);
    pi_null = pi_null_V(q);
[theta_qlr_pi, Q_FCP] = fmincon(@(beta)cf_beta(Y_up, Y_lag, pow, beta, pi_null), beta_true, [], [], [], [], b_L, b_H, [], opt);
    qlr_pi = n*(log(Q_FCP) - log(Q_unres));
    if qlr_pi<0 %This only happens rarely due to computational errors
        qlr_pi=0;
    end
    Q_FCP_V(q) = qlr_pi;
    t_FCP_V(q) = (pi_star - pi_null)/se_pi;
end


%% stack statistic together
A_F(k,:,z)=pre_t(1); 

Q_FCP_M(k,:,z) = Q_FCP_V;
t_FCP_M(k,:,z) = t_FCP_V;

end  % finish loop over b 
z = z+1;
end







